Unit uVistaFuncs;

Interface

Uses
  Forms, Windows, Graphics;

Function IsWindowsVista: Boolean;
Procedure SetVistaFonts(Const AForm: TCustomForm);
Procedure SetVistaContentFonts(Const AFont: TFont);
Procedure SetDesktopIconFonts(Const AFont: TFont);
Procedure ExtendGlass(Const AHandle: THandle; Const AMargins: TRect);
Function CompositingEnabled: Boolean;
Function TaskDialog(Const AHandle: THandle; Const ATitle, ADescription,
  AContent: String; Const Icon, Buttons: Integer): Integer;
Procedure SetVistaTreeView(Const AHandle: THandle);

Const
  VistaFont        = 'Segoe UI';
  VistaContentFont = 'Calibri';
  XPContentFont    = 'Verdana';
  XPFont           = 'Tahoma';

  TD_ICON_BLANK           = 0;
  TD_ICON_WARNING         = 84;
  TD_ICON_QUESTION        = 99;
  TD_ICON_ERROR           = 98;
  TD_ICON_INFORMATION     = 81;
  TD_ICON_SHIELD_QUESTION = 104;
  TD_ICON_SHIELD_ERROR    = 105;
  TD_ICON_SHIELD_OK       = 106;
  TD_ICON_SHIELD_WARNING  = 107;

  TD_BUTTON_OK     = 1;
  TD_BUTTON_YES    = 2;
  TD_BUTTON_NO     = 4;
  TD_BUTTON_CANCEL = 8;
  TD_BUTTON_RETRY  = 16;
  TD_BUTTON_CLOSE  = 32;

  TD_RESULT_OK     = 1;
  TD_RESULT_CANCEL = 2;
  TD_RESULT_RETRY  = 4;
  TD_RESULT_YES    = 6;
  TD_RESULT_NO     = 7;
  TD_RESULT_CLOSE  = 8;

Var
  CheckOSVerForFonts: Boolean = True;

Implementation

Uses
  SysUtils, Dialogs, Controls, UxTheme;

Procedure SetVistaTreeView(Const AHandle: THandle);
Begin
  If IsWindowsVista Then
    SetWindowTheme(AHandle, 'explorer', NIL);
End;

Procedure SetVistaFonts(Const AForm: TCustomForm);
Begin
  If (IsWindowsVista Or Not CheckOSVerForFonts) And Not SameText(AForm.Font.Name, VistaFont) And (Screen.Fonts.IndexOf(VistaFont) >= 0) Then
  Begin
    AForm.Font.Size := AForm.Font.Size + 1;
    AForm.Font.Name := VistaFont;
  End;
End;

Procedure SetVistaContentFonts(Const AFont: TFont);
Begin
  If (IsWindowsVista Or Not CheckOSVerForFonts) And Not SameText(AFont.Name, VistaContentFont) And (Screen.Fonts.IndexOf(VistaContentFont) >= 0) Then
  Begin
    AFont.Size := AFont.Size + 2;
    AFont.Name := VistaContentFont;
  End;
End;

Procedure SetDefaultFonts(Const AFont: TFont);
Begin
  AFont.Handle := GetStockObject(DEFAULT_GUI_FONT);
End;

Procedure SetDesktopIconFonts(Const AFont: TFont);
Var
  LogFont: TLogFont;
Begin
  If SystemParametersInfo(SPI_GETICONTITLELOGFONT, SizeOf(LogFont), @LogFont, 0) Then
    AFont.Handle := CreateFontIndirect(LogFont)
  Else
    SetDefaultFonts(AFont);
End;

Function IsWindowsVista: Boolean;
Var
  VerInfo: TOSVersioninfo;
Begin
  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(VerInfo);
  Result := VerInfo.dwMajorVersion >= 6;
End;

Const
  dwmapi                          = 'dwmapi.dll';
  DwmIsCompositionEnabledSig      = 'DwmIsCompositionEnabled';
  DwmExtendFrameIntoClientAreaSig = 'DwmExtendFrameIntoClientArea';
  TaskDialogSig                   = 'TaskDialog';

Function CompositingEnabled: Boolean;
Var
  DLLHandle:                                                  THandle;
  DwmIsCompositionEnabledProc:                                Function(pfEnabled: PBoolean): HRESULT; Stdcall;
  Enabled: Boolean;
Begin
  Result := False;
  If IsWindowsVista Then
  Begin
    DLLHandle := LoadLibrary(dwmapi);

    If DLLHandle <> 0 Then
    Begin
      @DwmIsCompositionEnabledProc := GetProcAddress(DLLHandle,
        DwmIsCompositionEnabledSig);

      If (@DwmIsCompositionEnabledProc <> NIL) Then
      Begin
        DwmIsCompositionEnabledProc(@Enabled);
        Result := Enabled;
      End;

      FreeLibrary(DLLHandle);
    End;
  End;
End;

//source: http://www.delphipraxis.net/topic93221,next.html
Procedure ExtendGlass(Const AHandle: THandle; Const AMargins: TRect);
Type
  _MARGINS = Packed Record
    cxLeftWidth:    Integer;
    cxRightWidth:   Integer;
    cyTopHeight:    Integer;
    cyBottomHeight: Integer;
  End;
  PMargins = ^_MARGINS;
  TMargins = _MARGINS;
Var
  DLLHandle:                                                                            THandle;
  DwmExtendFrameIntoClientAreaProc:                                                     Function(destWnd: HWND; Const pMarInset: PMargins): HRESULT; Stdcall;
  Margins: TMargins;
Begin
  If IsWindowsVista And CompositingEnabled Then
  Begin
    DLLHandle := LoadLibrary(dwmapi);

    If DLLHandle <> 0 Then
    Begin
      @DwmExtendFrameIntoClientAreaProc := GetProcAddress(DLLHandle,
        DwmExtendFrameIntoClientAreaSig);

      If (@DwmExtendFrameIntoClientAreaProc <> NIL) Then
      Begin
        ZeroMemory(@Margins, SizeOf(Margins));
        Margins.cxLeftWidth    := AMargins.Left;
        Margins.cxRightWidth   := AMargins.Right;
        Margins.cyTopHeight    := AMargins.Top;
        Margins.cyBottomHeight := AMargins.Bottom;

        DwmExtendFrameIntoClientAreaProc(AHandle, @Margins);
      End;

      FreeLibrary(DLLHandle);
    End;
  End;
End;

//source: http://www.tmssoftware.com/atbdev5.htm
Function TaskDialog(Const AHandle: THandle; Const ATitle, ADescription,
  AContent: String; Const Icon, Buttons: Integer): Integer;
Var
  DLLHandle:                      THandle;
  res:                            Integer;
  S:                              String;
  wTitle, wDescription, wContent: Array[0..1024] Of Widechar;
  Btns:                           TMsgDlgButtons;
  DlgType:                        TMsgDlgType;
  TaskDialogProc:                 Function(HWND: THandle; hInstance: THandle; cTitle, cDescription, cContent: pwidechar; Buttons: Integer; Icon: Integer;
    ResButton: pinteger): Integer; Cdecl Stdcall;
Begin
  Result := 0;
  If IsWindowsVista Then
  Begin
    DLLHandle := LoadLibrary(comctl32);
    If DLLHandle >= 32 Then
    Begin
      @TaskDialogProc := GetProcAddress(DLLHandle, TaskDialogSig);

      If Assigned(TaskDialogProc) Then
      Begin
        StringToWideChar(ATitle, wTitle, SizeOf(wTitle));
        StringToWideChar(ADescription, wDescription, SizeOf(wDescription));

        //Get rid of line breaks, may be here for backwards compat but not
        //needed with Task Dialogs
        {Suppression des sauts de ligne ; prsent pour compatibilit descendante mais non 
              ncessaire avec les Dialogues de Tche}
        S := StringReplace(AContent, #10, '', [rfReplaceAll]);
        S := StringReplace(S, #13, '', [rfReplaceAll]);
        StringToWideChar(S, wContent, SizeOf(wContent));

        TaskDialogProc(AHandle, 0, wTitle, wDescription, wContent, Buttons,
          Icon, @res);

        Result := mrOk;

        Case res Of
          TD_RESULT_CANCEL:
            Result := mrCancel;
          TD_RESULT_RETRY:
            Result := mrRetry;
          TD_RESULT_YES:
            Result := mrYes;
          TD_RESULT_NO:
            Result := mrNo;
          TD_RESULT_CLOSE:
            Result := mrAbort;
        End;
      End;
      FreeLibrary(DLLHandle);
    End;
  End
  Else
  Begin
    Btns := [];
    If Buttons And TD_BUTTON_OK = TD_BUTTON_OK Then
      Btns := Btns + [mbOK];

    If Buttons And TD_BUTTON_YES = TD_BUTTON_YES Then
      Btns := Btns + [mbYes];

    If Buttons And TD_BUTTON_NO = TD_BUTTON_NO Then
      Btns := Btns + [mbNo];

    If Buttons And TD_BUTTON_CANCEL = TD_BUTTON_CANCEL Then
      Btns := Btns + [mbCancel];

    If Buttons And TD_BUTTON_RETRY = TD_BUTTON_RETRY Then
      Btns := Btns + [mbRetry];

    If Buttons And TD_BUTTON_CLOSE = TD_BUTTON_CLOSE Then
      Btns := Btns + [mbAbort];

    DlgType := mtCustom;

    Case Icon Of
      TD_ICON_WARNING:
        DlgType := mtWarning;
      TD_ICON_QUESTION:
        DlgType := mtConfirmation;
      TD_ICON_ERROR:
        DlgType := mtError;
      TD_ICON_INFORMATION:
        DlgType := mtInformation;
    End;

    Result := MessageDlg(AContent, DlgType, Btns, 0);
  End;
End;

End.
